<?php namespace TierPricingTable;

use TierPricingTable\Settings\Settings;
use WC_Product;
use WC_Product_Variable;

/**
 * Class CatalogPriceManager
 *
 * @package TierPricingTable
 */
class CatalogPriceManager {

	/**
	 * Settings
	 *
	 * @var Settings
	 */
	private $settings;

	/**
	 * Price hash
	 *
	 * @var string
	 */
	private $variablePriceHash;

	/**
	 * CatalogPriceManager constructor.
	 *
	 * @param Settings $settings
	 */
	public function __construct( Settings $settings ) {
		$this->settings = $settings;

		if ( $this->isEnable() && ! is_admin() ) {
			add_filter( 'woocommerce_get_price_html', array( $this, 'formatPrice' ), 99, 2 );

			// Store variable products price hash
			add_filter( 'woocommerce_get_variation_prices_hash', function ( $hash, WC_Product_Variable $product ) {

				$hash[] = $product->get_date_modified();

				$this->variablePriceHash[ $product->get_id() ] = md5( wp_json_encode( $hash ) . $this->getDisplayType() );

				return $hash;
			}, 10, 2 );
		}
	}

	/**
	 * Change logic showing prince at catalog for product with tiered price rules
	 *
	 * @param string $priceHtml
	 * @param WC_Product $product
	 *
	 * @return string
	 */
	public function formatPrice( $priceHtml, $product ) {

		$currentProductPageProductId = get_queried_object_id();
		$parentProductId             = $product->is_type( 'variation' ) ? $product->get_parent_id() : $product->get_id();
		$isAdjustPriceOnProductPage  = 'yes' === $this->settings->get( 'tiered_price_at_product_page',
				'no' );

		if ( $currentProductPageProductId === $parentProductId ) {

			if ( $isAdjustPriceOnProductPage ) {
				if ( $product instanceof WC_Product_Variable ) {
					$newPriceHTML = $this->getFormattedPriceForVariableProduct( $product );
				} else {
					$newPriceHTML = $this->getFormattedPriceForSimpleProducts( $product );
				}
			} else {
				$newPriceHTML = false;
			}

		} else {

			if ( $product instanceof WC_Product_Variable && 'yes' === $this->useForVariable() ) {
				$newPriceHTML = $this->getFormattedPriceForVariableProduct( $product );
			} else {
				$newPriceHTML = $this->getFormattedPriceForSimpleProducts( $product );
			}
		}

		$newPriceHtml = $newPriceHTML ? $newPriceHTML . ' ' . $product->get_price_suffix() : $priceHtml;

		return apply_filters( 'tier_pricing_table/catalog_pricing/price_html', $newPriceHtml, $priceHtml, $product );
	}

	/**
	 * Format price for simple/variation products.
	 *
	 * @param WC_Product $product
	 *
	 * @return bool|string
	 */
	protected function getFormattedPriceForSimpleProducts( WC_Product $product ) {

		$priceHtml = false;

		if ( in_array( $product->get_type(), array( 'simple', 'variation' ) ) ) {

			$displayPriceType = $this->getDisplayType();
			$rules            = PriceManager::getPriceRules( $product->get_id() );

			if ( ! empty( $rules ) ) {
				if ( 'range' === $displayPriceType ) {
					$priceHtml = $this->getRange( $rules, $product ) . $product->get_price_suffix();
				} else {
					$priceHtml = $this->getLowestPrice( $rules, $product ) . $product->get_price_suffix();
				}
			}
		}

		return $priceHtml;
	}

	/**
	 * Format price for variable product. Range uses lowest and high prices from all variations
	 *
	 * @param WC_Product_Variable $product
	 * @param bool $force
	 *
	 * @return bool|string
	 */
	protected function getFormattedPriceForVariableProduct( WC_Product_Variable $product, $force = false ) {

		if ( ! $this->isCacheForVariableProductsEnabled() ) {
			return $this->formatPriceForVariableProduct( $product );
		}

		if ( ! empty( $this->variablePriceHash[ $product->get_id() ] ) ) {

			$priceKey = 'wc_var_tiered_price_html_' . $product->get_id() . $this->variablePriceHash[ $product->get_id() ];
			$cacheKey = 'tier_pricing_cached_prices';

			$value = (array) get_transient( $cacheKey );

			if ( ! $value || empty( $value[ $priceKey ] ) || $force ) {
				$priceHTML = $this->formatPriceForVariableProduct( $product );

				$value[ $priceKey ] = $priceHTML;

				set_transient( $cacheKey, $value, DAY_IN_SECONDS * 10 + rand( 10, DAY_IN_SECONDS ) );

				return $priceHTML;
			}

			return (string) $value[ $priceKey ];
		}

		return false;
	}

	/**
	 * Format price for variable product. Range uses lowest and high prices from all variations
	 *
	 * @param WC_Product_Variable $product
	 *
	 * @return bool|string
	 */
	protected function formatPriceForVariableProduct( WC_Product_Variable $product ) {

		// With taxes
		$maxPrice  = (float) $product->get_variation_price( 'max', true );
		$minPrices = array( (float) $product->get_variation_price( 'min', true ) );

		foreach ( $product->get_available_variations() as $variation ) {
			$rules = PriceManager::getPriceRules( $variation['variation_id'] );

			if ( ! empty( $rules ) ) {
				$minPrices[] = $this->getLowestPrice( $rules, wc_get_product( $variation['variation_id'] ), false );
			}
		}

		if ( ! empty( $minPrices ) && count( $minPrices ) > 1 ) {

			if ( 'range' === $this->getDisplayType() ) {

				if ( min( $minPrices ) === $maxPrice ) {
					return false;
				}

				return wc_price( min( $minPrices ) ) . ' - ' . wc_price( $maxPrice );
			} else {
				return $this->getLowestPrefix() . ' ' . wc_price( min( $minPrices ) );
			}
		}

		return false;
	}

	/**
	 * Get range from lowest to highest price from price rules
	 *
	 * @param array $rules
	 * @param WC_Product $product
	 *
	 * @return string
	 */
	protected function getRange( $rules, $product ) {

		$lowest = array_pop( $rules );

		$highest_html = wc_price( wc_get_price_to_display( $product, array(
			'price' => $product->get_price(),
		) ) );

		if ( 'percentage' === PriceManager::getPricingType( $product->get_id() ) ) {

			$lowest_html = wc_price(
				wc_get_price_to_display( $product, array(
					'price' => PriceManager::getPriceByPercentDiscount( $product->get_price(), $lowest ),
				) )
			);

			$range = $lowest_html . ' - ' . $highest_html;
		} else {
			$lowest_html = wc_price(
				wc_get_price_to_display( $product, array(
					'price' => $lowest,
				) )
			);

			$range = $lowest_html . ' - ' . $highest_html;
		}

		if ( $lowest_html !== $highest_html ) {
			return $range;
		}

		return $lowest_html;
	}

	/**
	 * Get lowest price from price rules
	 *
	 * @param array $rules
	 * @param WC_Product $product
	 *
	 * @param bool $html
	 *
	 * @return string|float
	 */
	protected function getLowestPrice( $rules, $product, $html = true ) {
		if ( 'percentage' === PriceManager::getPricingType( $product->get_id() ) ) {

			$lowest = PriceManager::getPriceByPercentDiscount( $product->get_price(),
				array_pop( $rules ) );
		} else {
			$lowest = array_pop( $rules );
		}

		if ( ! $html ) {
			return wc_get_price_to_display( $product, array(
				'price' => $lowest
			) );
		}

		return $this->getLowestPrefix() . ' ' . wc_price( wc_get_price_to_display( $product, array(
				'price' => $lowest
			) ) );
	}

	public function getLowestPrefix() {
		return $this->settings->get( 'lowest_prefix', __( 'From', 'tier-pricing-table' ) );
	}

	public function isEnable() {
		return 'yes' === $this->settings->get( 'tiered_price_at_catalog', 'yes' );
	}

	public function getDisplayType() {
		return $this->settings->get( 'tiered_price_at_catalog_type', 'range' );
	}

	public function useForVariable() {
		return $this->settings->get( 'tiered_price_at_catalog_for_variable', 'yes' );
	}

	public function isCacheForVariableProductsEnabled() {
		return $this->settings->get( 'tiered_price_at_catalog_cache_for_variable', 'no' ) === 'yes';
	}
}
